import 'package:flutter/material.dart';

typedef ColoredWidgetBuilder = Widget Function(BuildContext context, Color color);

class BlinkingAnimation with ChangeNotifier {
  /// Creates a [BlinkingAnimation] widget
  BlinkingAnimation({
    Key? key,
    Color? beginColor,
    Color? endColor,
    Duration? duration,
    this.repeats,
    this.disabled = false,
    required this.vsync,
  }) {
    this.beginColor = beginColor ?? Colors.black;
    this.endColor = endColor ?? Colors.transparent;
    this.duration = duration ?? Duration(milliseconds: 1000);
    color = this.beginColor;
    _initializeController();
  }

  final TickerProvider vsync;

  /// Duration of blinking animation, default is 1 second.
  late Duration duration;

  /// Times to repeat color animation. [null] is infinite, 0 is without animation.
 int? repeats;

  /// Starting color, default value is [Colors.black].
  late final Color beginColor;

  /// End color, default value is [Colors.transparent].
  late Color endColor;

  late Color color;
  // Disable animation. Build widget only once with beginColor.
  bool disabled;

  AnimationController? controller;
  late Animation<Color?> colorAnimation;
  int repeatCounter = 0;

  // @override
  // void didUpdateWidget(covariant BlinkingAnimation oldWidget) {
  //   super.didUpdateWidget(oldWidget);

  //   if (widget.disabled != oldWidget.disabled ||
  //       widget.repeats != oldWidget.repeats ||
  //       widget.endColor != oldWidget.endColor ||
  //       widget.duration != oldWidget.duration) {
  //     repeatCounter = 0;
  //     _initializeController();
  //   }
  // }

  void _initializeController() {
    print('_initializeController');

    if (disabled == true || repeats == 0) {
      controller?.stop();
      return;
    }

    // Initialize controller and animation
    controller?.dispose();
    controller = AnimationController(vsync: vsync, duration: duration);
    colorAnimation = ColorTween(
      begin: beginColor,
      end: endColor,
    ).animate(
      CurvedAnimation(
        parent: controller!,
        curve: Curves.linear,
      ),
    );

    controller!.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        repeatCounter++;
        controller!.reverse().then((value) {
          if (disabled || (repeats != null && repeatCounter >= repeats!)) {
            controller!.stop();
          }
        });
      } else if (status == AnimationStatus.dismissed) {
        controller!.forward();
      }
      color = colorAnimation.value!;
      notifyListeners();
    });

    controller!.forward();
  }

  void setDisabled(bool value) {
    disabled = value;
    _initializeController();
  }

  void setEndColor(Color? value) {
    endColor = value ?? Colors.transparent;
    _initializeController();
  }

  void setRepeats(int? value) {
    repeats = value;
    _initializeController();
  }

  void setDuration(Duration? value) {
    duration= value ?? Duration(milliseconds: 1000);
    _initializeController();
  }

  void dispose() {
    controller?.dispose();
    super.dispose();
  }

  // @override
  // Widget build(BuildContext context) {
  //   if (controller == null || disabled == true) {
  //     return widget.builder(context, beginColor);
  //   } else {
  //     return AnimatedBuilder(
  //       animation: controller!,
  //       builder: (context, child) => widget.builder(context, colorAnimation.value!),
  //     );
  //   }
  // }
}
